#ifndef __VECTOR__
#include "Vector.h"
#endif

#ifndef __ERRORS__
#include <errors.h>
#endif

#ifndef __STDLIB__
#include <stdlib.h>
#endif



//   Vector::Vector
#pragma segment Vector
Vector::Vector()
	: DataArea(),
	  fLength(0)
	{
	}



//   Vector::IVector
#pragma segment Vector
int Vector::IVector(size_t initialSize, size_t increment)
{
	fLength = 0;
	return (IDataArea(initialSize * sizeof(void *), increment * sizeof(void *)));
}




//   Vector::IVector
#pragma segment Vector
int Vector::IVector(const Vector *aVector)
{
	if (aVector == 0)
		return (memPCErr);
	fLength = aVector->fLength;
	return (IDataArea(aVector));
}




//   Vector::AtPut
#pragma segment Vector
void Vector::AtPut(int i, const void *anItem)
{
	if (i >= 0 && i < fLength)
		_AtPut(i, anItem);
}




//   Vector::IndexOf
#pragma segment Vector
int Vector::IndexOf(const void *anItem)
{
	int n = fLength;
	void **p = (void **)GetData(0);

	for (; n; ++p, --n)
		if (*p == anItem)
			return (fLength - n);

	return (-1);
}




//   Vector::AddOnce
#pragma segment Vector
Boolean Vector::AddOnce(const void *anItem)
{
	return ((IndexOf(anItem) >= 0) ? true : AddItem(anItem));
}




//   Vector::AddOnce
#pragma segment Vector
Boolean Vector::AddOnce(const Vector *aVector)
{
	for (int i = 0; i < aVector->Length(); i++)
		if (!AddOnce(aVector->At(i)))
			return (false);

	return (true);
}




//   Vector::AddItem
#pragma segment Vector
Boolean Vector::AddItem(const void *anItem)
{
	if (Require(sizeof(anItem)) >= sizeof(anItem)) {
		_AtPut(fLength++, anItem);
		IncrCursor(sizeof(anItem));
		return (true);
	}

	return (false);
}



//   Vector::Sort
#pragma segment Vector
void Vector::Sort(int (*aCompareFunc)(const void *, const void *))
{
	HLock();
	qsort(*GetHandle(), Length(), sizeof(void*), aCompareFunc);
	HUnlock();
}




//   Vector::MakeEmpty
#pragma segment Vector
void Vector::MakeEmpty()
{
	fLength = 0;
	SetCursor(0);
	Truncate();
}
